package de.uni_passau.fim.PADAS.group3.DataDash.Dataset;

import java.time.LocalDate;
import java.util.Optional;
import java.util.UUID;

import org.springframework.data.domain.Pageable;
import org.springframework.stereotype.Service;

import de.uni_passau.fim.PADAS.group3.DataDash.category.Category;
import de.uni_passau.fim.PADAS.group3.DataDash.category.CategoryRepository;

import org.springframework.data.domain.Page;

@Service
public class DatasetService {
    private dataRepository datasetRepository;
    private CategoryRepository categoryRepository;

    DatasetService(dataRepository datasetRepository, CategoryRepository categoryRepository) {
        this.datasetRepository = datasetRepository;
        this.categoryRepository = categoryRepository;
    }

    Dataset getDatasetById(UUID id) {
        return datasetRepository.getDatasetById(id);
    }

    Dataset addDataset(Dataset dataset) {
        dataset.setDate(LocalDate.now());
        return datasetRepository.save(dataset);
    }

    void voteDataset(UUID id, int vote) {
        Dataset dataset = datasetRepository.getDatasetById(id);
        dataset.vote(vote);
        datasetRepository.save(dataset);
    }

    void deleteDataset(UUID id) {
        Dataset dataset = datasetRepository.getDatasetById(id);
        datasetRepository.delete(dataset);
    }

    void upvoteDataset(UUID id) {
        Dataset dataset = datasetRepository.getDatasetById(id);
        dataset.upvote();
        datasetRepository.save(dataset);
    }

    void downvoteDataset(UUID id) {
        Dataset dataset = datasetRepository.getDatasetById(id);
        dataset.downvote();
        datasetRepository.save(dataset);
    }

    Page<Dataset> searchByOptionalCriteria(String search, String categories, String type, Pageable pageable) {
        Category category = categories.equals("%") ? null
                : categoryRepository.getCategoryById(UUID.fromString(categories));
        Type t = type.equals("%") ? null : Type.valueOf(type);

        if (category == null) {
            return datasetRepository.searchByOptionalCriteria(Optional.ofNullable(search), Optional.ofNullable(t),
                    pageable);
        }
        return datasetRepository.searchByOptionalCriteriaWithCategory(Optional.ofNullable(search), category,
                Optional.ofNullable(t), pageable);
    }

}